/*
 * CCBigImage
 *
 * Cocos2D-iPhone-Extensions v0.2.1
 * https://github.com/cocos2d/cocos2d-iphone-extensions
 *
 * Copyright (c) 2010-2012 Stepan Generalov
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#import "CCTextureCache+CCBigImageExtensions.h"

#if COCOS2D_VERSION < 0x00020000

@interface CCAsyncObject : NSObject
{
	SEL			selector_;
	id			target_;
	id			data_;
}
@property	(readwrite,assign)	SEL			selector;
@property	(readwrite,strong)	id			target;
@property	(readwrite,strong)	id			data;
@end

@interface CCTextureCache (CCAsyncObject)

-(void) addImageWithAsyncObject:(CCAsyncObject*)async;

@end

#endif

@implementation CCTextureCache (iTraceurDynamicTiles)

-(void) addImageFromAnotherThreadWithName: (NSString*) filename target:(id)target selector:(SEL)selector
{
	NSAssert(filename != nil, @"TextureCache: fileimage MUST not be nill");
	
	// load here async 	
#if COCOS2D_VERSION >= 0x00020000
    [self addImageAsync:filename target:target selector:selector];
#else
	CCAsyncObject *asyncObject = [[ CCAsyncObject alloc] init];
	asyncObject.selector = selector;
	asyncObject.target = target;
	asyncObject.data = filename;
	
	[self addImageWithAsyncObject: asyncObject];
	[asyncObject release];
#endif
    
}

@end

@implementation CCTexture2D (Extension)

- (void) setMipMapTexParameters
{
	//use mip map only on Mac - with enough VRAM
#ifdef __MAC_OS_X_VERSION_MIN_REQUIRED
	[self generateMipmap];
	ccTexParams texParams = { GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE };	
	[self setTexParameters:&texParams];
#endif
}

@end
